## Poor man's interpolation
# JF Moyen July 2018

library(akima)
library(ggplot2)
library(scales)
library(gridExtra)

readWerTab<-function(fn,first.line,last.line,consolidate=T,fix.names=T){
  #' Read and format a tab file
  #' fn: filename, full (i.e. foo_1.tab)
  #' first.line, last.line: first and last data lines (NOT headers)
  #' consolidate: add athe amounts of same-name phases
  #' fix names: correct colnames (using make.names() to safe ones)
  
  # Read the file and parse
  ee<-scan(fn,what=character(),blank.lines.skip=F,sep="\n")
  hdr<-unlist(strsplit(ee[first.line-1]," +"))
  
  qq<-strsplit(ee[first.line:last.line]," +")
  foo<-array(unlist(qq),dim=c(length(hdr)+1,length(qq)))
  foo<-foo[-1,]
  foo<-apply(foo,1,as.numeric)
  colnames(foo)<-hdr
  
  # Combine duplicates
  
  if(consolidate){
    bar<-NULL
    foo[is.na(foo)]<-0
    for(i in unique(hdr)){
      ph<-apply(foo[,hdr==i,drop=F],1,sum)
      bar<-cbind(bar,ph)
      colnames(bar)[ncol(bar)]<-i
    }
    bar[bar==0]<-NA
  }else{
    bar<-foo
  }
  
  if(fix.names){
    colnames(bar)<-make.names(colnames(bar))
  }
  
  return(bar)
}

prepareData<-function(basename,first.line,last.line,P.bar.=NULL){
  #' Combine two tab files (_1.tab and _2.tab, not flexible !!) into a matrix
  #' basename: filename, no extensions (i.e foo)
  #' first.line, last.line: first and last data lines (NOT headers)
  #' P.bar.: must be specified for a T-X, otherwise leave null for a T-P
  
  # The files
  fn1<-paste(basename,"1.tab",sep="_")
  fn2<-paste(basename,"2.tab",sep="_")
  
  # Read two tab files and remove the common cols: they should be P and T
  mat1<-readWerTab(fn1,first.line,last.line,T,T)
  mat2<-readWerTab(fn2,first.line,last.line,T,T)
  
  already.there<-intersect(colnames(mat1),colnames(mat2))
  mat2<-mat2[,setdiff(colnames(mat2),already.there)]
  
  dd<-cbind(mat1,mat2)
  if(!is.null(P.bar.)){
    dd<-cbind(dd,P.bar.)
  }
  # Useful variables
  T.C<-dd[,"T.K."]-273
  P.kbar<-dd[,"P.bar."]/1000
  Hm.J.kg<-dd[,"H.J.mol"]/dd[,"N.g"]*1000
  Hv.J.m<-Hm.J.kg*dd[,"rho.kg.m3"]  
  Cpm.J.K.kg<-dd[,"cp.J.K.mol"]/dd[,"N.g"]*1000
  dd<-cbind(dd,Hm.J.kg,Hv.J.m,T.C,P.kbar,Cpm.J.K.kg)
  
  return(dd)
  
}

Rwerami<-function(ds,xvar,yvar,zvar,tvar=zvar,
                  interpolate=F,nx=50,ny=50,
                  xlab=xvar,ylab=yvar,zlab=zvar,tlab=tvar,
                  col=c("yellow","red"),na.col="white",fill=T){
  
  #' ds; a werami dataset (see prepareData)
  #' xvar, yvar: x and y variables, i.e. the plotting axes
  #' zvar: the variable used for colouring
  #' tvar: a variable used to add additional contours on the graph, if required
  #' xlab, ylab, zlab, tlab: how the variables will be named in the axes/captions
  #' interpolate: boolean. If T, the z- ad t-values will be interpolated on the grid defined
  #' by x and y. Required if x and y do not defined a regularly spaced grid, i.e. useless 
  #' for P and T but needed for H, etc.
  #' nx, ny: number of cells for the interpolation (grid size). 
  #' fill: if T, the fill properties are defined here using the following parameters (col and na.col).
  #' Otherwise they are ignored. In this case they should be set further down after calling the function
  #' (I know, it's an ugly hack).
  #' col: a colour palette, can be defined in different ways -- 
  #' essentially it must be a vector containing valid colour names or codes. Many functions allow
  #' to define one (see Rcolorbrewer for instance, or the function gmtColors here.)
  #' na.col: colour for missing values
  
 
  
   xlims<-range(ds[,xvar],na.rm=T)
  ylims<-range(ds[,yvar],na.rm=T)
  
  dd<- as.data.frame(ds[,c(xvar,yvar,zvar,tvar)])
  names(dd)<-c("x","y","z","t")
  if(interpolate){ # for 2 variables, z and t, allows to have contours and fill
    
    # Rescale systematically before interpolating...
    xx<-rescale(dd$x)
    yy<-rescale(dd$y)
    
    ddR<-data.frame(x=xx,y=yy,z=dd$z,t=dd$t)
    
    # interpolate on z
    idxz<-is.na(ddR[,"z"])
    dd2<-ddR[!idxz,]
    
    ez<-with(dd2,interp(x=x,y=y,z=z,xo=seq(0,1,length.out=nx),yo=seq(0,1,length.out=ny),nx=nx,ny=ny))
    eez<-interp2xyz(ez,data.frame=T)
    
    # interpolate on t
    idxt<-is.na(ddR[,"t"])
    dd3<-ddR[!idxt,]
    
    et<-with(dd3,interp(x=x,y=y,z=t,xo=seq(0,1,length.out=nx),yo=seq(0,1,length.out=ny),nx=nx,ny=ny))
    eet<-interp2xyz(et,data.frame=T)
    
    # Stitch the pieces back together
    out<-data.frame(x=rescale(eez$x,xlims),y=rescale(eez$y,ylims),z=eez$z,t=eet$z)
    
    dd<-out
    
  }
  v<-ggplot(dd)+aes(x=x,y=y,z=z,fill=z)+geom_tile()+
    scale_x_continuous(name=xlab,expand=c(0,0))+
    scale_y_continuous(name=ylab,expand=c(0,0))
  if(fill){v<-v+
    scale_fill_gradientn(colours=col,na.value = na.col, guide=guide_colourbar(title=zlab))
  }

  return(v)
}

gmtColors <- function(pal.name="relief"){
  #' Code stolen from http://www.r-bloggers.com/gmt-standard-color-palettes/
  #' palette name `pal.name` is one of the following:
  #' "cool", "copper", "gebco", "globe", "gray",
  #' "haxby", "hot", "jet", "no_green", "ocean",
  #' "polar", "rainbow", "red2green", "relief", "sealand",
  #' "seis", "split", "topo", "wysiwyg"
  
  tmp <- structure(list(cool = c("#00FFFF", "#0DF2FF", "#19E6FF", "#26D9FF", 
                                 "#33CCFF", "#3FBFFF", "#4CB3FF", "#59A6FF", "#6699FF", "#738CFF", 
                                 "#7F7FFF", "#8C73FF", "#9966FF", "#A659FF", "#B24DFF", "#BF3FFF", 
                                 "#CC33FF", "#D926FF", "#E619FF", "#F20DFF"), copper = c("#000000", 
                                                                                         "#100906", "#1F130D", "#301E13", "#40281A", "#50321F", "#603C26", 
                                                                                         "#70462D", "#805033", "#905A3A", "#A06440", "#B06E46", "#C0784D", 
                                                                                         "#D08253", "#E08C5A", "#F09660", "#FFA066", "#FFAA6D", "#FFB473", 
                                                                                         "#FFBE7A"), gebco = c("#00F0FF", "#00F0FF", "#00F0FF", "#23FFFF", 
                                                                                                               "#23FFFF", "#23FFFF", "#5AFFFF", "#5AFFFF", "#5AFFFF", "#8CFFE6", 
                                                                                                               "#8CFFE6", "#8CFFE6", "#A5FFD7", "#A5FFD7", "#A5FFD7", "#C3FFD7", 
                                                                                                               "#C3FFD7", "#C3FFD7", "#D2FFD7", "#E6FFF0"), globe = c("#9900FF", 
                                                                                                                                                                      "#9900FF", "#7722FF", "#5544FF", "#3366FF", "#1188FF", "#1BA4FF", 
                                                                                                                                                                      "#51BAFF", "#86D0FF", "#BCE6FF", "#336600", "#F3CA89", "#D9A627", 
                                                                                                                                                                      "#A49019", "#9F7B0D", "#996600", "#B27676", "#C2B0B0", "#E5E5E5", 
                                                                                                                                                                      "#FFFFFF"), gray = c("#000000", "#0D0D0D", "#191919", "#262626", 
                                                                                                                                                                                           "#333333", "#3F3F3F", "#4C4C4C", "#595959", "#666666", "#737373", 
                                                                                                                                                                                           "#7F7F7F", "#8C8C8C", "#999999", "#A6A6A6", "#B2B2B2", "#BFBFBF", 
                                                                                                                                                                                           "#CCCCCC", "#D9D9D9", "#E6E6E6", "#F2F2F2"), haxby = c("#090079", 
                                                                                                                                                                                                                                                  "#280096", "#0009C8", "#0019D4", "#1A66F0", "#19AFFF", "#32BEFF", 
                                                                                                                                                                                                                                                  "#61E1F0", "#6AECE1", "#8AECAE", "#CDFFA2", "#DFF68D", "#F8D768", 
                                                                                                                                                                                                                                                  "#FFBD57", "#F4754B", "#FF5A5A", "#FF7C7C", "#F6B3AE", "#FFC4C4", 
                                                                                                                                                                                                                                                  "#FFECEC"), hot = c("#000000", "#220000", "#440000", "#660000", 
                                                                                                                                                                                                                                                                      "#880000", "#AA0000", "#CC0000", "#EE0000", "#FF1100", "#FF3300", 
                                                                                                                                                                                                                                                                      "#FF5500", "#FF7700", "#FF9900", "#FFBB00", "#FFDD00", "#FFFF00", 
                                                                                                                                                                                                                                                                      "#FFFF33", "#FFFF66", "#FFFF99", "#FFFFCC"), jet = c("#00007F", 
                                                                                                                                                                                                                                                                                                                           "#0000B2", "#0000E5", "#0019FF", "#004DFF", "#007FFF", "#00B2FF", 
                                                                                                                                                                                                                                                                                                                           "#00E5FF", "#FFFFF2", "#FFFFD9", "#FFFFBF", "#FFFFA5", "#FFFF8C", 
                                                                                                                                                                                                                                                                                                                           "#FFE500", "#FFB300", "#FF7F00", "#FF4C00", "#FF1900", "#E50000", 
                                                                                                                                                                                                                                                                                                                           "#B20000"), no_green = c("#1F60FF", "#1F60FF", "#1F9FFF", "#1FBFFF", 
                                                                                                                                                                                                                                                                                                                                                    "#00CFFF", "#2AFFFF", "#2AFFFF", "#55FFFF", "#7FFFFF", "#AAFFFF", 
                                                                                                                                                                                                                                                                                                                                                    "#FFFF54", "#FFFF54", "#FFF000", "#FFBF00", "#FFA800", "#FF8A00", 
                                                                                                                                                                                                                                                                                                                                                    "#FF8A00", "#FF7000", "#FF4D00", "#FF0000"), ocean = c("#000000", 
                                                                                                                                                                                                                                                                                                                                                                                                           "#000209", "#000413", "#00061E", "#000728", "#000932", "#002650", 
                                                                                                                                                                                                                                                                                                                                                                                                           "#00426E", "#005E8C", "#007AAA", "#0096C8", "#22A9C2", "#45BCBB", 
                                                                                                                                                                                                                                                                                                                                                                                                           "#67CFB5", "#8AE2AE", "#ACF6A8", "#BCF8B9", "#CBF9CA", "#DBFBDC", 
                                                                                                                                                                                                                                                                                                                                                                                                           "#EBFDED"), polar = c("#0000FF", "#1919FF", "#3333FF", "#4C4CFF", 
                                                                                                                                                                                                                                                                                                                                                                                                                                 "#6666FF", "#7F7FFF", "#9999FF", "#B2B2FF", "#CCCCFF", "#E6E6FF", 
                                                                                                                                                                                                                                                                                                                                                                                                                                 "#FFFFFF", "#FFE5E5", "#FFCCCC", "#FFB2B2", "#FF9999", "#FF7F7F", 
                                                                                                                                                                                                                                                                                                                                                                                                                                 "#FF6666", "#FF4C4C", "#FF3333", "#FF1A1A"), rainbow = c("#FF00FF", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          "#BF00FF", "#7F00FF", "#3F00FF", "#0000FF", "#003FFF", "#007FFF", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          "#00BFFF", "#00FFFF", "#00FFBF", "#00FF7F", "#00FF3F", "#00FF00", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          "#3FFF00", "#7FFF00", "#BFFF00", "#FFFF00", "#FFBF00", "#FF7F00", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          "#FF3F00"), red2green = c("#FF0000", "#FF1919", "#FF3333", "#FF4C4C", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    "#FF6666", "#FF7F7F", "#FF9999", "#FFB2B2", "#FFCCCC", "#FFE6E6", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    "#FFFFFF", "#E5FFE5", "#CCFFCC", "#B2FFB2", "#99FF99", "#7FFF7F", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    "#66FF66", "#4CFF4C", "#33FF33", "#1AFF1A"), relief = c("#000000", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            "#000413", "#000728", "#002650", "#005E8C", "#0096C8", "#45BCBB", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            "#8AE2AE", "#BCF8B9", "#DBFBDC", "#467832", "#887438", "#B19D48", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            "#DBC758", "#FAE769", "#FAEB7E", "#FCED93", "#FCF1A7", "#FCF6C1", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            "#FDFAE0"), sealand = c("#8C66FF", "#6A66FF", "#6684FF", "#66A7FF", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    "#66CAFF", "#66ECFF", "#66FFF0", "#66FFCE", "#66FFAB", "#66FF88", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    "#66FF66", "#88FF66", "#ABFF66", "#CEFF66", "#FFEEA6", "#FFD3A6", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    "#FFB8A6", "#FFAAB0", "#FFB5CB", "#FFC0E1"), seis = c("#AA0000", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          "#D00000", "#F70000", "#FF1D00", "#FF4400", "#FF6A00", "#FF9000", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          "#FFB700", "#FFDD00", "#FFFF00", "#FFFF00", "#FFFF00", "#BDFF0C", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          "#73FF1A", "#3FFA36", "#16F45A", "#00D08B", "#0087CD", "#0048FA", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          "#0024E3"), split = c("#7F7FFF", "#6666E6", "#4D4DCC", "#3333B3", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                "#1A1A99", "#00007F", "#000066", "#00004D", "#000033", "#00001A", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                "#000000", "#1A0000", "#330000", "#4D0000", "#660000", "#7F0000", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                "#991A1A", "#B33333", "#CC4D4D", "#E66666"), topo = c("#C977D9", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      "#A18AE6", "#8AA2E6", "#8BD1E7", "#8AF3CF", "#85F38E", "#BDF385", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      "#EDE485", "#F0B086", "#DE9F8B", "#74A3B3", "#99CC70", "#DCD68E", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      "#EDDFAD", "#F7E8CA", "#FFF9F3", "#FFF9F6", "#FFFBF9", "#FFFCFA", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      "#FFFEFD"), wysiwyg = c("#3F003F", "#3F003F", "#3F00BF", "#003FFF", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              "#00A0FF", "#3FBFFF", "#3FBFFF", "#40E0FF", "#3FFFBF", "#3FFF3F", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              "#7FFF3F", "#BFFF3F", "#BFFF3F", "#FFE040", "#FFE040", "#FF6040", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              "#FF1F40", "#FF60C0", "#FFA0FF", "#FFA0FF")), .Names = c("cool", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       "copper", "gebco", "globe", "gray", "haxby", "hot", "jet", "no_green", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       "ocean", "polar", "rainbow", "red2green", "relief", "sealand", 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       "seis", "split", "topo", "wysiwyg"))
  
  tmp[[match(pal.name, names(tmp))]]
}

# werami file format
fl<-14
ll<-10013

# Prepare data
cev<-prepareData("CEV",fl,ll)

### Figures showing the H-T-F relationships for CEV

# Calculate a kJ value, nicer than J...
Hk<-cev[,"Hm.J.kg"]/1000
cev1<-cbind(cev,Hk)

windows(8,11.5) #A4 size
v1<-Rwerami(cev1,"T.C","P.kbar","Hk","melt.HP.",interpolate=T,nx=100,ny=100,
            xlab="T (°C)",ylab="P (kbar)",zlab="H (kJ/kg)",col=gmtColors("haxby"))+ 
  geom_contour(bins=10,aes(x,y,z=z/1000), color="white", size=0.4)+
  geom_contour(breaks=c(1,10,20,30,40,49),aes(x,y,z=t), color="darkslategrey", size=0.6)+
  theme(panel.border = element_rect(colour = "black", fill=NA, size=1))
# print(v1) ## Direct mode

v2<-Rwerami(cev1,"Hk","P.kbar","T.C","melt.HP.",interpolate=T,nx=100,ny=100,
            zlab="T (°C)",ylab="P (kbar)",xlab="H (kJ/kg)",na.col="grey60")+ 
  geom_contour(breaks=c(200,300,400,500,600,700,800,900,1000),aes(x,y,z=z), color="white", size=0.4)+
  geom_contour(breaks=c(1,10,20,30,40,49),aes(x,y,z=t), color="darkslategrey", size=0.6)+
  theme(panel.border = element_rect(colour = "black", fill=NA, size=1))
# print(v2) 

grid.arrange(v1,v2)
